#    Highly extensible Dependency injection framework for humans. Django integration
#    Copyright (C) 2017 Dmitriy Selischev
#    The MIT License (MIT)
#    
#    Permission is hereby granted, free of charge, to any person obtaining
#    a copy of this software and associated documentation files
#    (the "Software"), to deal in the Software without restriction,
#    including without limitation the rights to use, copy, modify, merge,
#    publish, distribute, sublicense, and/or sell copies of the Software,
#    and to permit persons to whom the Software is furnished to do so,
#    subject to the following conditions:
#    
#    The above copyright notice and this permission notice shall be
#    included in all copies or substantial portions of the Software.
#    
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
#    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
#    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import abc
import typing
from hexdi.utils import load_module, get_module_if_exists, load_class
from djhexdi import settings


def load_modules(modules, excluded_modules):
    loader = load_class(settings.HEXDI_LOADER)([m for m in modules if m not in excluded_modules])
    loader.load()


def find_modules(packages):
    finder = load_class(settings.HEXDI_FINDER)(packages)
    return finder.find()


class AbstractStrategy(abc.ABC):
    @abc.abstractmethod
    def go(self): pass


class Dummy(AbstractStrategy):
    def go(self):
        pass


class AbstractLoadModulesStrategy(AbstractStrategy):
    @abc.abstractmethod
    def discover_modules(self) -> typing.List[typing.AnyStr]:
        pass

    def get_excluded_modules(self) -> typing.List[typing.AnyStr]:
        return settings.HEXDI_EXCLUDE_MODULES

    def go(self):
        modules = self.discover_modules()
        excluded = self.get_excluded_modules()
        load_modules(modules, excluded)


class Static(AbstractLoadModulesStrategy):

    def discover_modules(self) -> typing.List[typing.AnyStr]:
        modules = list()
        file = settings.HEXDI_MODULE
        if file and get_module_if_exists(file):
            _module = load_module(file)
            modules.extend(getattr(_module, settings.HEXDI_MODULE_LIST_NAME))
        modules.extend(settings.HEXDI_MODULES_LIST)
        return modules


class Auto(Static):
    def discover_modules(self) -> typing.List[typing.AnyStr]:
        packages = settings.HEXDI_FINDER_PACKAGES
        modules = find_modules(packages)
        static_modules = super(Auto, self).discover_modules()
        modules.extend(static_modules)
        return modules



#    Highly extensible Dependency injection framework for humans. Django integration
#    Copyright (C) 2017 Dmitriy Selischev
#    The MIT License (MIT)
#    
#    Permission is hereby granted, free of charge, to any person obtaining
#    a copy of this software and associated documentation files
#    (the "Software"), to deal in the Software without restriction,
#    including without limitation the rights to use, copy, modify, merge,
#    publish, distribute, sublicense, and/or sell copies of the Software,
#    and to permit persons to whom the Software is furnished to do so,
#    subject to the following conditions:
#    
#    The above copyright notice and this permission notice shall be
#    included in all copies or substantial portions of the Software.
#    
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
#    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
#    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import os
import hexdi
from django.core.management import BaseCommand
from djhexdi import settings


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('-m',
                            dest='modulepath',
                            metavar='modulepath',
                            default=settings.HEXDI_MODULE,
                            type=str,
                            help='Optional. Dotted path to module that stores finder results.\n' +
                                 'Used HEXDI_MODULE({}) setting by default.'.format(settings.HEXDI_MODULE))
        parser.add_argument('--auto',
                            const=True,
                            default=False,
                            nargs='?',
                            metavar='',
                            help='Optional. Automatically create packages tree for module.')

    def handle(self, *args, **options):
        finder = hexdi.get_finder(settings.HEXDI_FINDER_PACKAGES)
        modules = finder.find()
        modules_text = "{} = [\n{}\n]".format(
            settings.HEXDI_MODULE_LIST_NAME,
            ",\n".join(["    \'{}\'".format(m) for m in modules])
        )
        raw_module_name = options['modulepath']
        if raw_module_name:
            filename = self.__sanitinze_hexdi_module_name(raw_module_name)
            self.__check_dir_exists_for_module(filename, options['auto'])
            with open(filename, 'w', encoding='utf8') as file:
                file.write(modules_text)
        else:
            self.stdout.write(modules_text)

    def __check_dir_exists_for_module(self, filename, auto_create):
        if '/' in filename:
            path, module_name = filename.rsplit('/', 1)
        else:
            path = ''
        path_parts = path.split('/')
        path_acc = ''
        for part in path_parts:
            path_acc += "{}/".format(part)
            if not os.path.isdir(path_acc):
                if auto_create:
                    os.mkdir(path_acc)
                    init_module_path = "{}__init__.py".format(path_acc)
                    with open(init_module_path, 'w', encoding='utf8'):
                        pass
                else:
                    raise Exception("No such path: {}".format(path))

    def __sanitinze_hexdi_module_name(self, name: str):
        if not name.endswith('.py'):
            _path = name.replace('.', '/')
        else:
            _path, _py = name.rsplit('.', 1)
            _path = _path.replace('.', '/')
        return "{}.py".format(_path)



import os
from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the README file
with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django-hexdi',
    version='0.2.1b1',
    packages=find_packages(include=('djhexdi', 'djhexdi.management', 'djhexdi.management.commands',), ),
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Application Frameworks',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here.
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    python_requires='>=3.5,<4',
    url='https://github.com/zibertscrem/django-hexdi',
    license='MIT',
    author='Dmitriy Selischev',
    author_email='zibertscrem@gmail.com',
    description='Highly extensible Dependency injection framework for humans. Django integration',
    long_description=long_description,
    install_requires=['typing', 'hexdi==0.2.1']
)



#    Highly extensible Dependency injection framework for humans. Django integration
#    Copyright (C) 2017 Dmitriy Selischev
#    The MIT License (MIT)
#    
#    Permission is hereby granted, free of charge, to any person obtaining
#    a copy of this software and associated documentation files
#    (the "Software"), to deal in the Software without restriction,
#    including without limitation the rights to use, copy, modify, merge,
#    publish, distribute, sublicense, and/or sell copies of the Software,
#    and to permit persons to whom the Software is furnished to do so,
#    subject to the following conditions:
#    
#    The above copyright notice and this permission notice shall be
#    included in all copies or substantial portions of the Software.
#    
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
#    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
#    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
#    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from django.test import TestCase

# Create your tests here.
